// The *nan() functions are replacements for the isnan() implemented in math.h.
// This is necessary as the library functions provided by the various RTEs 
// do not comply with IEEE and cannot deal with SNaN.
//
// A nice and concise explanation of the NaN topic can be found here:
// http://stevehollasch.com/cgindex/coding/ieeefloat.html

#include <sys/types.h>

static u_int64_t hf = 0x0004000000000000; // Highest bit of the fraction set
static u_int64_t em = 0x7FF8000000000000; // All exponent bits are 1
static u_int64_t fm = 0x0007FFFFFFFFFFFF; // All bits are set

// IEEE compliant processors are X86, X86-64, Itanium, Sparc, HP-Alpha, 
// IBM-Power
int ieee_isnan(double d)
{
    u_int64_t v = *(u_int64_t*)&d;
    return (((em & v) == em) && ((fm & v) !=0));
}

int ieee_isSnan(double d)
{
    u_int64_t v = *(u_int64_t*)&d;
#if defined(IEEE)
    return ( ieee_isnan(d) && ((hf & v) == 0));
#elif (defined PA-RISC) || (defined MIPS)
    return (ieee_isnan(d) && ((hf & v) != 0));
#else
// give an error at compile time
#error Define the NaN for the new platform
#endif
}

int ieee_isQnan(double d)
{
    u_int64_t v =*(u_int64_t*)&d;
#if defined(IEEE)
    return (ieee_isnan(d) && ((hf & v) != 0));
#elif (defined PA-RISC) || (defined MIPS)
    return (ieee_isnan(d) && ((hf & v) == 0));
#else
// give an error at compile time
#error Define the NaN for the new platform
#endif
}

